<?php

/**
 * @file
 * The page of Link to us links.
 */

/**
 * \brief Display the Link to us page.
 *
 * \todo
 * Add support for views
 *
 * \todo
 * Make the link appear on term pages!
 *
 * This function is the one that creates the page of text and banner links.
 */
function link_to_us_controller() {
  global $base_url;

  // get the arguments, we skip arg(0), it is /link_to/
  $args = array();
  for ($i = 1;; ++$i) {
    $arg = arg($i);
    if (empty($arg)) {
      break;
    }
    $args[] = $arg;
  }
  $url = implode("/", $args);

  // $i is one more than expected here... (it starts at 1)
  if ($i == 3 && $args[0] == 'node' && is_numeric($args[1])) {
    $type = 'node';
    $object = node_load(array('nid' => $args[1]));
    $path = drupal_get_path_alias($url);
    $title = db_result(db_query("SELECT title FROM {node} WHERE nid = %d", $args[1]));
  }
  elseif ($i == 4 && $args[0] == 'taxonomy' && $args[1] == 'term' && is_numeric($args[2])) {
    $type = 'term';
    $path = drupal_get_path_alias($url);
    $object = taxonomy_get_term($args[2]);
    $title = $object->name;
  }
  else {
    // default to the front page which is a node
    $type = 'unknown';
    $object = NULL;
    $url = '<front>';
    $path = drupal_get_path_alias($url);
    // XXX: we could also get the title of the front page if it is a node?
    $title = variable_get('site_name', 'Drupal');
  }
  $title = check_plain($title);

  drupal_set_title(t('Link to !title', array('!title' => $title)));

  // Create an absolute path since this will be displayed on other sites.
  $path = url($path, array('absolute' => TRUE));

  if (variable_get('link_to_us_textlink', FALSE)) {
    $text_help = variable_get('link_to_us_text_help', link_to_us_get_text_help_text());
    if (!$text_help) {
      $text_help = link_to_us_get_text_help_text();
    }
    $links[] = array(
      'help' => check_plain($text_help),
      'link' => '<a href="' . $path . '" title="' . $title . '">' . $title . '</a>',
      'banner' => $title,
      'id' => 'link_to_us_text_link',
      'url' => $path, // WARNING: This is correct, it is NOT $url
      'title' => $title,
      'path' => $path,
    );
  }

  $help = check_plain(variable_get('link_to_us_banner_help', link_to_us_get_banner_help_text()));
  if (!$help) {
    $help = check_plain(link_to_us_get_banner_help_text());
  }
  $images = link_to_us_image_selects();
  $count = 1;
  foreach ($images as $imagepath => $image) {
    // XXX
    // The following is not 100% correct because
    // 1. two files can have different extensions and yet have the same name
    // 2. two files can use different case and have the same name (Linux only)
    // So to be 100% safe we should probably check with an array that it is indeed unique
    $id = basename($imagepath);
    $id = preg_replace(array('/\.[^\.]*$/', '/-/'), array('', '_'), $id);
    if (!$id) {
      $id = 'id' . $count;
      $count++;
    }
    $id = 'link_to_us_' . check_plain(drupal_strtolower($id));

    // determine the URL to the image without the url() call because the
    // url() generates a '?q=' which does not work correctly for images.
    // (that's in the case when clean URLs are not working)
    $options = array('prefix' => '');
    $url = $imagepath;
    if (function_exists('language_url_rewrite')) {
      language_url_rewrite($url, $options);
    }
    $url = $base_url . '/' . drupal_urlencode($options['prefix'] . $url);
    $banner = '<img src="' . $url . '" alt="' . $title . '" title="' . $title . '" />';

    $links[] = array(
      'help' => $help,
      'link' => '<a href="' . $path . '" title="' . $title . '">' . $banner . '</a>',
      'banner' => $banner,
      'id' => $id,
      'url' => $url,
      'title' => $title,
      'path' => $path,
    );
  }

  return theme('link_to_us_banners', $links, $type, $object);
}

/**
 * \brief Theme the page of links.
 *
 * This function can be overridden by your theme in order to get links
 * to you page with a somewhat different model.
 *
 * The \p $links parameter is an array of links. Each link includes the
 * following information:
 *
 * \li $link['id'] The id of this specific link.
 * \li $link['help'] The help string.
 * \li $link['banner'] The banner if this is an image link.
 * \li $link['link'] The text link if this is not an image link.
 * \li $link['url'] The absolute URL to the banner (if text link, same as $link['path'])
 * \li $link['title'] The title by itself
 * \li $link['path'] The absolute URL to the page being worked on
 *
 * \param[in] $links An array of links.
 * \param[in] $type The type of page being linked to ('node', 'term', etc.)
 * \param[in] $objects The object corresponding to \p $type
 */
function theme_link_to_us_banners($links, $type, $object) {
  $output = '<div class="content link-to-us">';

  // add the node teaser?
  if ($type == 'node' && variable_get('link_to_us_show_node_teaser', TRUE)) {
    $output .= '<div class="link-to-us-node-teaser">'
      . node_view($object, TRUE, FALSE, FALSE)
      . '</div>';
  }

  $output .= '<div class="link-to-us-header">'
    . check_markup(variable_get('link_to_us_header', link_to_us_get_header_text()))
    . '</div>';

  if (count($links)) {
    $textarea_resizable = FALSE;
    $textarea = variable_get('link_to_us_textarea', '60x3');
    if (!$textarea) {
      $textarea = '60x3';
    }
    $textarea_sizes = explode('x', $textarea);
    switch (count($textarea_sizes)) {
    default:
      $textarea_width = 60;
      $textarea_height = 3;
      break;

    case 1:
      $textarea_width = (int)$textarea_sizes[0];
      $textarea_height = 1;
      break;

    case 2:
      $textarea_width = (int)$textarea_sizes[0];
      $textarea_height = (int)$textarea_sizes[1];
      if (substr($textarea_sizes[1], -1) == 'r') {
        $textarea_resizable = TRUE;
      }
      break;

    }
    if ($textarea_width < 12) {
      $textarea_width = 12;
    }
    if ($textarea_height < 1) {
      $textarea_height = 1;
    }
    $use_textarea = $textarea_height == 1;
    $links_output = '';
    foreach ($links as $link) {
      $links_output .= '<div class="link-to-us-link-block">';
      if ($link['help'] != '&lt;none&gt;') {
        $links_output .= '<div class="link-to-us-banner-help">' . $link['help'] . ':</div>';
      }
      $links_output .= '<div class="link-to-us-banner-link">' . $link['link'] . '</div>';
      if ($textarea_height == 1) {
        $links_output .= '<input size="60" readonly="readonly" id="' . $link['id'] . '"'
            . ' type="text" class="link-to-us-link" tabindex="400"'
            . ' value="' . check_plain($link['link']) . '"/>';
      }
      else {
        $textarea = array(
          '#parents' => array(0 => 'ignore'),
          '#required' => FALSE,
          '#name' => 'name_' . $link['id'], // it is not a form per se, so the name is not important
          '#id' => $link['id'],
          '#cols' => $textarea_width,
          '#rows' => $textarea_height,
          '#value' => $link['link'],
          '#wysiwyg' => FALSE,
          '#resizable' => $textarea_resizable,
          '#attributes' => array('class' => 'link-to-us-link', 'readonly' => 'readonly'),
        );
        $links_output .= theme('textarea', $textarea);
      }
      $links_output .= '</div>';
    }

    // put the result in a box so we can avoid leakage on the side on narrow websites
    // (although you may want to make smaller banners in that case?!)
    $max_width = (int)variable_get('link_to_us_max_width', 0);
    if ($max_width > 0) {
      if ($max_width < 150) {
        $max_width = 150;
      }
      $links_output = '<div class="link-to-us-box" style="max-width: '
        . $max_width . 'px;">' . $links_output . '</div>';
    }

    if (variable_get('link_to_us_fieldset', FALSE)) {
      $fieldset = array(
        '#type' => 'fieldset',
        '#title' => t('Link to us'),
        '#value' => $links_output,
        '#attributes' => array('class' => 'link-to-us-fieldset'),
        '#collapsible' => TRUE,
      );
      $links_output = theme('fieldset', $fieldset);
    }
    $output .= $links_output;
  }

  $output .= '<div class="link-to-us-header">'
    . check_markup(variable_get('link_to_us_footer', link_to_us_get_footer_text()))
    . '</div>';

  // closing content/link-to-us block
  $output .= '</div>';

  return $output;
}

// vim: ts=2 sw=2 et syntax=php
